@@ -29,12 +29,12 @@
 
 /* NoOp */
 
-void 
-bus_drop_all_directory_watches (void)
-{
-}
-
 void
 bus_watch_directory (const char *dir, BusContext *context)
 {
 }
+
+void
+bus_set_watched_dirs (BusContext *context, DBusList **directories)
+{
+}
