@@ -65,9 +65,6 @@ signal_handler (int sig)
       }
       break;
 #endif
-    case SIGTERM:
-      _dbus_loop_quit (bus_context_get_loop (context));
-      break;
     }
 }
 
@@ -156,7 +153,10 @@ handle_reload_watch (DBusWatch    *watch,
 {
   DBusError error;
   DBusString str;
-  _dbus_string_init (&str);
+
+  while (!_dbus_string_init (&str))
+    _dbus_wait_for_memory ();
+
   if ((reload_pipe[RELOAD_READ_END] > 0) &&
       _dbus_read_socket (reload_pipe[RELOAD_READ_END], &str, 1) != 1)
     {
@@ -464,7 +464,6 @@ main (int argc, char **argv)
 #ifdef SIGHUP
   _dbus_set_signal_handler (SIGHUP, signal_handler);
 #endif
-  _dbus_set_signal_handler (SIGTERM, signal_handler);
 #ifdef DBUS_BUS_ENABLE_DNOTIFY_ON_LINUX 
   _dbus_set_signal_handler (SIGIO, signal_handler);
 #endif /* DBUS_BUS_ENABLE_DNOTIFY_ON_LINUX */
