@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -939,6 +939,7 @@ _dbus_message_test (const char *test_data_dir)
   unsigned char v_BYTE;
   unsigned char v2_BYTE;
   dbus_bool_t v_BOOLEAN;
+  DBusMessageIter iter, array_iter, struct_iter;
 
   message = dbus_message_new_method_call ("org.freedesktop.DBus.TestService",
                                           "/org/freedesktop/TestPath",
@@ -949,7 +950,7 @@ _dbus_message_test (const char *test_data_dir)
                                              "TestMethod"));
   _dbus_assert (strcmp (dbus_message_get_path (message),
                         "/org/freedesktop/TestPath") == 0);
-  _dbus_message_set_serial (message, 1234);
+  dbus_message_set_serial (message, 1234);
 
   /* string length including nul byte not a multiple of 4 */
   if (!dbus_message_set_sender (message, "org.foo.bar1"))
@@ -1041,7 +1042,7 @@ _dbus_message_test (const char *test_data_dir)
                                           "/org/freedesktop/TestPath",
                                           "Foo.TestInterface",
                                           "TestMethod");
-  _dbus_message_set_serial (message, 1);
+  dbus_message_set_serial (message, 1);
   dbus_message_set_reply_serial (message, 5678);
 
   v_INT16 = -0x123;
@@ -1172,7 +1173,7 @@ _dbus_message_test (const char *test_data_dir)
   dbus_message_unref (copy);
 
   /* Message loader test */
-  _dbus_message_lock (message);
+  dbus_message_lock (message);
   loader = _dbus_message_loader_new ();
   
   /* check ref/unref */
@@ -1226,6 +1227,7 @@ _dbus_message_test (const char *test_data_dir)
       DBusError error = DBUS_ERROR_INIT;
       char *marshalled = NULL;
       int len = 0;
+      char garbage_header[DBUS_MINIMUM_HEADER_SIZE] = "xxx";
 
       if (!dbus_message_marshal (message, &marshalled, &len))
         _dbus_assert_not_reached ("failed to marshal message");
@@ -1233,6 +1235,7 @@ _dbus_message_test (const char *test_data_dir)
       _dbus_assert (len != 0);
       _dbus_assert (marshalled != NULL);
 
+      _dbus_assert (dbus_message_demarshal_bytes_needed (marshalled, len) == len);
       message2 = dbus_message_demarshal (marshalled, len, &error);
 
       _dbus_assert (message2 != NULL);
@@ -1255,6 +1258,14 @@ _dbus_message_test (const char *test_data_dir)
       _dbus_assert (message2 == NULL);
       _dbus_assert (dbus_error_is_set (&error));
       dbus_error_free (&error);
+
+      /* Bytes needed to demarshal empty message: 0 (more) */
+
+      _dbus_assert (dbus_message_demarshal_bytes_needed ("", 0) == 0);
+      
+      /* Bytes needed to demarshal invalid message: -1 (error). */
+
+      _dbus_assert (dbus_message_demarshal_bytes_needed (garbage_header, DBUS_MINIMUM_HEADER_SIZE) == -1);
     }
 
   dbus_message_unref (message);
@@ -1262,6 +1273,34 @@ _dbus_message_test (const char *test_data_dir)
 
   check_memleaks ();
 
+  /* Check that we can abandon a container */
+  message = dbus_message_new_method_call ("org.freedesktop.DBus.TestService",
+		  			  "/org/freedesktop/TestPath",
+					  "Foo.TestInterface",
+					  "Method");
+
+  dbus_message_iter_init_append (message, &iter);
+
+  _dbus_assert (dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY,
+			  			  (DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+						   DBUS_TYPE_STRING_AS_STRING
+						   DBUS_TYPE_STRING_AS_STRING
+						   DBUS_STRUCT_END_CHAR_AS_STRING),
+						  &array_iter));
+  _dbus_assert (dbus_message_iter_open_container (&array_iter, DBUS_TYPE_STRUCT,
+			  			  NULL, &struct_iter));
+
+  s = "peaches";
+  _dbus_assert (dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_STRING,
+			  			&s));
+
+  /* uh-oh, error, try and unwind */
+
+  dbus_message_iter_abandon_container (&array_iter, &struct_iter);
+  dbus_message_iter_abandon_container (&array_iter, &iter);
+
+  dbus_message_unref (message);
+
   /* Load all the sample messages from the message factory */
   {
     DBusMessageDataIter diter;
