@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-userdb.c User database abstraction
  * 
  * Copyright (C) 2003, 2004  Red Hat, Inc.
@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #define DBUS_USERDB_INCLUDES_PRIVATE 1
@@ -143,7 +143,7 @@ _dbus_user_database_lookup (DBusUserDatabase *db,
         uid = n;
     }
 
-#ifdef DBUS_ENABLE_USER_CACHE  
+#ifdef DBUS_ENABLE_USERDB_CACHE  
   if (uid != DBUS_UID_UNSET)
     info = _dbus_hash_table_lookup_ulong (db->users, uid);
   else
@@ -226,7 +226,8 @@ static DBusString process_homedir;
 static void
 shutdown_system_db (void *data)
 {
-  _dbus_user_database_unref (system_db);
+  if (system_db != NULL)
+    _dbus_user_database_unref (system_db);
   system_db = NULL;
   _dbus_string_free (&process_username);
   _dbus_string_free (&process_homedir);
@@ -239,15 +240,13 @@ init_system_db (void)
     
   if (system_db == NULL)
     {
-      DBusError error;
+      DBusError error = DBUS_ERROR_INIT;
       const DBusUserInfo *info;
       
       system_db = _dbus_user_database_new ();
       if (system_db == NULL)
         return FALSE;
 
-      dbus_error_init (&error);
-
       if (!_dbus_user_database_get_uid (system_db,
                                         _dbus_getuid (),
                                         &info,
@@ -347,7 +346,8 @@ _dbus_user_database_flush_system (void)
 {
   _dbus_user_database_lock_system ();
    
-  _dbus_user_database_flush (system_db);
+   if (system_db != NULL)
+    _dbus_user_database_flush (system_db);
 
   _dbus_user_database_unlock_system ();
 }
